﻿Imports Infragistics.Win.DataVisualization
Imports System.ComponentModel
Public Class Form1

    Const CATEGORYITEMCOUNT As Integer = 10

    Private Sub Form1_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        ResetState()
    End Sub


    Private Sub ResetState()
        Me.UltraDataChart1.Axes.Clear()
        Me.UltraDataChart1.Series.Clear()
        Me.cboPropGridSelector.Items.Clear()

        Me.cboPropGridSelector.SelectedIndex = Me.cboPropGridSelector.Items.Add(New ComboItem(Me.UltraDataChart1))

        Dim seriesCrosshair = New CrosshairLayer()
        Me.UltraDataChart1.Series.Add(seriesCrosshair)
    End Sub


    'Helper Control Events
    Private Sub btnClear_Click(sender As Object, e As EventArgs) Handles btnClear.Click
        ResetState()
    End Sub
    Private Sub btnAddScatterLine_Click(sender As Object, e As EventArgs) Handles btnAddScatterLine.Click
        Dim series = FillScatterLineSeries(10)
        series.MarkerType = MarkerType.Circle
        series.IsHighlightingEnabled = True

        Me.UltraDataChart1.Series.Add(series)
    End Sub
    Private Sub btnAddLine_Click(sender As Object, e As EventArgs) Handles btnAddLine.Click
        Dim series = FillLineSeries(10)
        series.MarkerType = MarkerType.Circle
        series.IsHighlightingEnabled = True

        Me.UltraDataChart1.Series.Add(series)
    End Sub
    Private Sub cboPropGridSelector_SelectedIndexChanged(sender As Object, e As EventArgs) Handles cboPropGridSelector.SelectedIndexChanged
        Dim item = DirectCast(Me.cboPropGridSelector.SelectedItem, ComboItem)
        Me.PropertyGrid1.SelectedObject = item.Item
    End Sub


    'Series Creation Methods
    Private Function FillScatterLineSeries(seriesCount As Integer) As ScatterLineSeries
        Dim retVal = New ScatterLineSeries()

        Dim data = New BindingList(Of ScatterItem)()
        Dim rand = New Random(Me.UltraDataChart1.Series.Count)

        For i As Integer = 0 To seriesCount
            data.Add(New ScatterItem(rand.Next(40, 80) / 4, rand.Next(40, 80) / 4, String.Format("Item  {0}", i)))
        Next

        Dim xAxis As NumericXAxis
        Dim yAxis As NumericYAxis

        'Find Axes if they exist
        For Each Axis In Me.UltraDataChart1.Axes
            If TypeOf Axis Is NumericXAxis Then
                xAxis = Axis
            End If
            If TypeOf Axis Is NumericYAxis Then
                yAxis = Axis
            End If
        Next

        'If Axes not found create them
        If IsNothing(xAxis) Then
            xAxis = New NumericXAxis()
            xAxis.Name = "NumericXAxis"
            Me.UltraDataChart1.Axes.Add(xAxis)
            Me.cboPropGridSelector.Items.Add(New ComboItem(xAxis))
        End If
        If IsNothing(yAxis) Then
            yAxis = New NumericYAxis()
            yAxis.Name = "NumericYAxis"
            Me.UltraDataChart1.Axes.Add(yAxis)
            Me.cboPropGridSelector.Items.Add(New ComboItem(yAxis))
        End If

        retVal.DataSource = data
        retVal.XMemberPath = "X"
        retVal.YMemberPath = "Y"
        retVal.XAxis = xAxis
        retVal.YAxis = yAxis
        retVal.Name = String.Format("ScatterLineSeries # {0}", ScatterLineSeriesCount + 1)

        Me.cboPropGridSelector.Items.Add(New ComboItem(retVal))

        Return retVal
    End Function
    Private Function FillLineSeries(seriesCount As Integer) As LineSeries
        Dim retVal = New LineSeries()

        Dim data = New BindingList(Of CategoryItem)()
        Dim rand = New Random(Me.UltraDataChart1.Series.Count)

        For i As Integer = 0 To seriesCount - 1
            data.Add(New CategoryItem(rand.Next(40, 80) / 4, CategoryHelper.Labels(i)))
        Next

        Dim xAxis As CategoryXAxis
        Dim yAxis As NumericYAxis

        'Find Axes if they exist
        For Each Axis In Me.UltraDataChart1.Axes
            If TypeOf Axis Is CategoryXAxis Then
                xAxis = Axis
            End If
            If TypeOf Axis Is NumericYAxis Then
                yAxis = Axis
            End If
        Next

        'If Axes not found create them
        If IsNothing(xAxis) Then
            xAxis = New CategoryXAxis()
            Dim list = New BindingList(Of ValuelessCategoryItem)()

            For x As Integer = 0 To CATEGORYITEMCOUNT - 1
                list.Add(New ValuelessCategoryItem(CategoryHelper.Labels(x)))
            Next

            xAxis.DataSource = list
            xAxis.Label = "Label"
            xAxis.Name = "CategoryXAxis"
            Me.UltraDataChart1.Axes.Add(xAxis)
            Me.cboPropGridSelector.Items.Add(New ComboItem(xAxis))
        End If
        If IsNothing(yAxis) Then
            yAxis = New NumericYAxis()
            yAxis.Name = "NumericYAxis"
            Me.UltraDataChart1.Axes.Add(yAxis)
            Me.cboPropGridSelector.Items.Add(New ComboItem(yAxis))
        End If

        retVal.DataSource = data
        retVal.ValueMemberPath = "Value"
        retVal.XAxis = xAxis
        retVal.YAxis = yAxis
        retVal.Name = String.Format("LineSeries # {0}", LineSeriesCount + 1)

        Me.cboPropGridSelector.Items.Add(New ComboItem(retVal))

        Return retVal
    End Function


    'Helper Properties used for Naming Series
    Protected ReadOnly Property LineSeriesCount As Integer
        Get
            Dim retVal = 0
            For Each series In Me.UltraDataChart1.Series
                If TypeOf (series) Is LineSeries Then
                    retVal = retVal + 1
                End If
            Next
            Return retVal
        End Get
    End Property
    Protected ReadOnly Property ScatterLineSeriesCount As Integer
        Get
            Dim retVal = 0
            For Each series In Me.UltraDataChart1.Series
                If TypeOf (series) Is ScatterLineSeries Then
                    retVal = retVal + 1
                End If
            Next
            Return retVal
        End Get
    End Property


    'Helper Class used to populate the PropertyGrid Selector Combobox
    Protected Class ComboItem
        Public ReadOnly Property Text As String
        Public ReadOnly Property Item As Object
        Sub New(control As Control)
            Me.Item = control
            Me.Text = control.Name
        End Sub
        Sub New(series As Series)
            Me.Item = series
            Me.Text = series.Name
        End Sub
        Sub New(axis As Axis)
            Me.Item = axis
            Me.Text = axis.Name
        End Sub
        Public Overrides Function ToString() As String
            Return text
        End Function
    End Class


    'DataHelper classes used to populate the series
    Protected Class ScatterItem
        Sub New(ByVal x As Double, ByVal y As Double, ByVal label As String)
            Me.X = x
            Me.Y = y
            Me.Label = label
        End Sub
        Public Property Label As String
        Public Property X As Double
        Public Property Y As Double
        Public Overrides Function ToString() As String
            Return String.Format("{0} [{1},{2}]", Me.Label, Me.X, Me.Y)
        End Function
    End Class
    Protected Class CategoryItem
        Sub New(ByVal value As Double, ByVal label As String)
            Me.Value = value
            Me.Label = label
        End Sub
        Public Property Label As String
        Public Property Value As Double
        Public Overrides Function ToString() As String
            Return String.Format("{0} [{1}]", Me.Label, Me.Value)
        End Function
    End Class
    Public Class ValuelessCategoryItem
        Public Property Label As String
        Sub New(s As String)
            Me.Label = s
        End Sub

    End Class
    Public Class CategoryHelper
        Private Shared _labels As List(Of String)
        Public Shared ReadOnly Property Labels As List(Of String)
            Get
                If IsNothing(_labels) Then
                    _labels = New List(Of String)()
                    For x = 0 To CATEGORYITEMCOUNT - 1
                        _labels.Add(Chr(x + 65).ToString())
                    Next
                End If

                Return _labels

            End Get
        End Property
    End Class

End Class
